@@ -122,16 +122,8 @@ impl Connection {
     }
 
     pub fn reset(&mut self) -> Result<(), Error> {
-        sleep(Duration::from_millis(100));
-
-        self.serial.write_data_terminal_ready(false)?;
-        self.serial.write_request_to_send(true)?;
-
-        sleep(Duration::from_millis(100));
-
-        self.serial.write_request_to_send(false)?;
-
-        Ok(())
+        let pid = self.port_info.pid;
+        Ok(reset_after_flash(&mut *self.serial, pid)?)
     }
 
     pub fn reset_to_flash(&mut self, extra_delay: bool) -> Result<(), Error> {
@@ -280,4 +272,35 @@ impl Connection {
     pub fn into_serial(self) -> Box<dyn SerialPort> {
         self.serial
     }
+
+    pub fn get_usb_pid(&self) -> Result<u16, Error> {
+        Ok(self.port_info.pid)
+    }
+}
+
+pub fn reset_after_flash(serial: &mut dyn SerialPort, pid: u16) -> Result<(), serialport::Error> {
+    sleep(Duration::from_millis(100));
+
+    if pid == USB_SERIAL_JTAG_PID {
+        serial.write_data_terminal_ready(false)?;
+
+        sleep(Duration::from_millis(100));
+
+        serial.write_request_to_send(true)?;
+        serial.write_data_terminal_ready(false)?;
+        serial.write_request_to_send(true)?;
+
+        sleep(Duration::from_millis(100));
+
+        serial.write_request_to_send(false)?;
+    } else {
+        serial.write_data_terminal_ready(false)?;
+        serial.write_request_to_send(true)?;
+
+        sleep(Duration::from_millis(100));
+
+        serial.write_request_to_send(false)?;
+    }
+
+    Ok(())
 }
