@@ -1,8 +1,8 @@
 pub use chip::Chip;
 pub use cli::config::Config;
-pub use elf::FirmwareImage;
+pub use elf::{FlashFrequency, FlashMode};
 pub use error::Error;
-pub use flasher::Flasher;
+pub use flasher::{FlashSize, Flasher};
 pub use image_format::ImageFormatId;
 pub use partition_table::PartitionTable;
 
